#' do_fit

#' do_fit
#' @param service service
#' @param algorithm algorithm in character
#' @param ... other parame related to fitting the model
#' @param dataset tibble or data.frame
#' @param col_bind col_bind in character
#' @param target target in character
#' @param predictors predictors in character
#' @param train train percentage
#' @param test test percentage
#' @param MODEL_DIR boolean save model in a specific recoverable folder, temporary solution
#' @keywords algoli
#' @export
#' @examples
#' \dontrun{
#' do_fit()
#' }
do_fit <- function(service,algorithm=NULL,...,dataset,col_bind=NULL,target,predictors, train=0.8,test=0.2,MODEL_DIR=FALSE){

  fileCheck_path <- file.path(getwd(),"algoli")
  fileCheck_path <- normalizePath(fileCheck_path,mustWork = FALSE)
  dir.create(fileCheck_path, showWarnings = TRUE, recursive = TRUE)
  fileCheck <- file.path(fileCheck_path,"fit-check.log")
  file.create(fileCheck)

  target <- gsub("\\.","_",target)
  predictors <- gsub("\\.","_",predictors)

  # new algoli
  hello <- algoli::algdoor(service = service)

  # Connecting to session ...
  cat(c(as.character(Sys.time()),"connecting\n"), file=fileCheck,append=TRUE)
  hello$connect()

  # Incoporating dataset into session  ...
  cat(c(as.character(Sys.time()),"copying and splitting dataset\n"), file=fileCheck,append=TRUE)
  hello$dataset(dataset=dataset,col_bind= col_bind, train=train,test=test)

  # Fitting the model on the training dataset ...
  cat(c(as.character(Sys.time()),"fit model on training\n"), file=fileCheck,append=TRUE)
  res_fit <- hello$fit(algorithm=algorithm,target=target,predictors=predictors,...)

  # exporting model ...
  cat(c(as.character(Sys.time()),"export model\n"), file=fileCheck,append=TRUE)
  tmp_model_path <- normalizePath(file.path(tempdir(),"model"),mustWork = FALSE)
  dir.create(tmp_model_path, showWarnings = TRUE, recursive = TRUE)
  from <- hello$export_model(res_fit,tmp_model_path)

  if (!MODEL_DIR) {
     tmp <- list(id=NULL,model_path=from)
  } else if (MODEL_DIR) {
      # for an internal access to the binary, temporary solution
    tmp <- dir_tmp_model()
    if (!is.null(tmp$model_path)){
      to <- file.path(tmp$model_path,"model.mod")
      file.copy(from=from,to=to)
    } else tmp <- list(id=NULL,model_path=NULL)
  } else tmp <- list(id=NULL,model_path=NULL)


  # Prediction on the test dataset ...
  cat(c(as.character(Sys.time()),"predict\n"), file=fileCheck,append=TRUE)
  res_pred <- hello$predict(res_fit)

  res <- list(service = service,
              alg  = algorithm,
              args = list(...),
              target = target,
              predictors = predictors,
              pred = res_pred,
              id = tmp$id,
              path = tmp$model_path)

  # disconnect ...
  cat(c(as.character(Sys.time()),"disconnect\n"), file=fileCheck,append=TRUE)
  hello$disconnect()

  return(res)
}
